@@ -47,10 +47,12 @@ server(Ancestors, Drv, Shell, Options) ->
     put(line_buffer, proplists:get_value(line_buffer, Options, group_history:load())),
     put(read_mode, list),
     put(user_drv, Drv),
-    put(unicode_state, true),
     ExpandFun = normalize_expand_fun(Options, fun edlin_expand:expand/2),
     put(expand_fun, ExpandFun),
-    put(echo, proplists:get_value(echo, Options, true)),
+    Echo = proplists:get_value(echo, Options, true),
+    put(echo, Echo),
+    Dumb = proplists:get_value(dumb, Options, false),
+    put(dumb, Dumb),
     put(expand_below, proplists:get_value(expand_below, Options, true)),
 
     server_loop(Drv, start_shell(Shell), []).
@@ -217,8 +219,7 @@ io_request(Req, From, ReplyAs, Drv, Shell, Buf0) ->
 
 
 %% Put_chars, unicode is the normal message, characters are always in
-%%standard unicode
-%% format.
+%% standard unicode format.
 %% You might be tempted to send binaries unchecked, but the driver
 %% expects unicode, so that is what we should send...
 %% io_request({put_chars,unicode,Binary}, Drv, Buf) when is_binary(Binary) ->
@@ -250,27 +251,17 @@ io_request({put_chars,unicode,M,F,As}, Drv, _Shell, From, Buf) ->
 	    end
     end;
 io_request({put_chars,latin1,Binary}, Drv, _Shell, From, Buf) when is_binary(Binary) ->
-    IsUnicode = get(unicode_state),
-    if IsUnicode ->
-            send_drv(Drv,
-                     {put_chars_sync, unicode,
-                      unicode:characters_to_binary(Binary,latin1),
-                      From});
-        true ->
-            send_drv(Drv, {put_chars_sync, latin1, Binary, From})
-    end,
+    send_drv(Drv, {put_chars_sync, unicode,
+                   unicode:characters_to_binary(Binary,latin1),
+                   From}),
     {noreply,Buf};
 io_request({put_chars,latin1,Chars}, Drv, _Shell, From, Buf) ->
-    IsUnicode = get(unicode_state),
-    if IsUnicode ->
-            case catch unicode:characters_to_binary(Chars,latin1) of
-                Binary when is_binary(Binary) ->
-                    send_drv(Drv, {put_chars_sync, unicode, Binary, From}),
-                    {noreply,Buf};
-                _ ->
-                    {error,{error,{put_chars,latin1,Chars}},Buf}
-            end;
-        true -> send_drv(Drv, {put_chars_sync, latin1, Chars, From})
+    case catch unicode:characters_to_binary(Chars,latin1) of
+        Binary when is_binary(Binary) ->
+            send_drv(Drv, {put_chars_sync, unicode, Binary, From}),
+            {noreply,Buf};
+        _ ->
+            {error,{error,{put_chars,latin1,Chars}},Buf}
     end;
 io_request({put_chars,latin1,M,F,As}, Drv, _Shell, From, Buf) ->
     case catch apply(M, F, As) of
@@ -401,13 +392,16 @@ setopts(Opts0,Drv,Buf) ->
     end.
 check_valid_opts([]) ->
     true;
-check_valid_opts([{binary,_}|T]) ->
+check_valid_opts([{binary,Flag}|T]) when is_boolean(Flag) ->
     check_valid_opts(T);
-check_valid_opts([{encoding,Valid}|T]) when Valid =:= unicode; Valid =:= utf8; Valid =:= latin1 ->
+check_valid_opts([{encoding,Valid}|T]) when Valid =:= unicode;
+                                            Valid =:= utf8;
+                                            Valid =:= latin1 ->
     check_valid_opts(T);
-check_valid_opts([{echo,_}|T]) ->
+check_valid_opts([{echo,Flag}|T]) when is_boolean(Flag) ->
     check_valid_opts(T);
-check_valid_opts([{expand_fun,_}|T]) ->
+check_valid_opts([{expand_fun,Fun}|T]) when is_function(Fun, 1);
+                                            is_function(Fun, 2) ->
     check_valid_opts(T);
 check_valid_opts(_) ->
     false.
@@ -415,14 +409,12 @@ check_valid_opts(_) ->
 do_setopts(Opts, Drv, Buf) ->
     put(expand_fun, normalize_expand_fun(Opts, get(expand_fun))),
     put(echo, proplists:get_value(echo, Opts, get(echo))),
-    case proplists:get_value(encoding,Opts) of
+    case proplists:get_value(encoding, Opts) of
 	Valid when Valid =:= unicode; Valid =:= utf8 ->
-           set_unicode_state(Drv,true),
-           put(unicode_state, true);
+           set_unicode_state(Drv,true);
 	latin1 ->
-           set_unicode_state(Drv,false),
-           put(unicode_state, false);
-	_ ->
+           set_unicode_state(Drv,false);
+	undefined ->
 	    ok
     end,
     case proplists:get_value(binary, Opts, case get(read_mode) of
@@ -434,8 +426,6 @@ do_setopts(Opts, Drv, Buf) ->
 	    {ok,ok,Buf};
 	false ->
 	    put(read_mode, list),
-	    {ok,ok,Buf};
-	_ ->
 	    {ok,ok,Buf}
     end.
 
@@ -503,19 +493,20 @@ get_chars_line(Prompt, M, F, Xa, Drv, Shell, Buf, Encoding) ->
     get_chars_loop(Pbs, M, F, Xa, Drv, Shell, Buf, start, [], Encoding).
 
 get_chars_loop(Pbs, M, F, Xa, Drv, Shell, Buf0, State, LineCont0, Encoding) ->
-    Result = case get(echo) of
+    Result = case not(get(dumb)) andalso get(echo) of
                  true ->
                      get_line(Buf0, Pbs, LineCont0, Drv, Shell, Encoding);
                  false ->
                      %% get_line_echo_off only deals with lists,
                      %% so convert to list before calling it.
-                     get_line_echo_off(cast(Buf0, list, Encoding), Pbs, Drv, Shell)
+                     get_line_echo_off(cast(Buf0, list), Encoding, Pbs, Drv, Shell)
              end,
     case Result of
         {done,LineCont1,Buf} ->
             get_chars_apply(Pbs, M, F, Xa, Drv, Shell, append(Buf, [], Encoding),
                             State, LineCont1, Encoding);
-
+        {no_translation, unicode, latin1} ->
+            {error,{error,{no_translation, unicode, latin1}}, []};
         interrupted ->
             {error,{error,interrupted},[]};
         terminated ->
@@ -525,16 +516,25 @@ get_chars_loop(Pbs, M, F, Xa, Drv, Shell, Buf0, State, LineCont0, Encoding) ->
 get_chars_apply(Pbs, M, F, Xa, Drv, Shell, Buf, State0, LineCont, Encoding) ->
     %% multi line support means that we should not keep the state
     %% but we need to keep it for oldshell mode
-    {State, Line} = case get(echo) of
+    {State, Line} = case not(get(dumb)) andalso get(echo) of
                         true -> {start, edlin:current_line(LineCont)};
                         false -> {State0, LineCont}
                     end,
     case catch M:F(State, cast(Line,get(read_mode), Encoding), Encoding, Xa) of
+        {stop,eof,_} ->
+            {ok,eof,eof};
         {stop,Result,eof} ->
             {ok,Result,eof};
         {stop,Result,Rest} ->
             %% Prompt was valid expression, clear the prompt in user_drv
-            send_drv_reqs(Drv, [new_prompt]),
+            %% First redraw without the multi line prompt
+            case LineCont of
+                {[CL|LB], _, _} ->
+                    LineCont1 = {LB,{lists:reverse(CL++"\n"), []},[]},
+                    MultiLinePrompt = lists:duplicate(prim_tty:npwcwidthstring(Pbs), $\s),
+                    send_drv_reqs(Drv, [{redraw_prompt, Pbs, MultiLinePrompt, LineCont1},new_prompt]);
+                _ -> skip %% oldshell mode
+            end,
             _ = case {M,F} of
                     {io_lib, get_until} ->
                         save_line_buffer(string:trim(Line, both)++"\n", get_lines(new_stack(get(line_buffer))));
@@ -549,20 +549,27 @@ get_chars_apply(Pbs, M, F, Xa, Drv, Shell, Buf, State0, LineCont, Encoding) ->
     end.
 
 get_chars_n_loop(Pbs, M, F, Xa, Drv, Shell, Buf0, State, Encoding) ->
-    try M:F(State, cast(Buf0, get(read_mode), Encoding), Encoding, Xa) of
-        {stop,Result,Rest} ->
-            {ok, Result, append(Rest,[],Encoding)};
-        State1 ->
-            case get_chars_echo_off(Pbs, Drv, Shell) of
-                interrupted ->
-                    {error,{error,interrupted},[]};
-                terminated ->
-                    {exit,terminated};
-                Buf ->
-                    get_chars_n_loop(Pbs, M, F, Xa, Drv, Shell, Buf, State1, Encoding)
+    case check_encoding(Buf0, Encoding) of
+        false ->
+            {error,{error,{no_translation,unicode,Encoding}},[]};
+        true ->
+            try M:F(State, cast(Buf0, get(read_mode), Encoding), Encoding, Xa) of
+                {stop,eof,_} ->
+                    {ok, eof, eof};
+                {stop,Result,Rest} ->
+                    {ok, Result, append(Rest,[],Encoding)};
+                State1 ->
+                    case get_chars_echo_off(Pbs, Drv, Shell) of
+                        interrupted ->
+                            {error,{error,interrupted},[]};
+                        terminated ->
+                            {exit,terminated};
+                        Buf ->
+                            get_chars_n_loop(Pbs, M, F, Xa, Drv, Shell, Buf, State1, Encoding)
+                    end
+            catch _:_ ->
+                    {error,{error,err_func(M, F, Xa)},[]}
             end
-    catch _:_ ->
-            {error,{error,err_func(M, F, Xa)},[]}
     end.
 
 %% Convert error code to make it look as before
@@ -589,23 +596,19 @@ get_line(Chars, Pbs, Cont, Drv, Shell, Encoding) ->
 get_line1({done, Cont, Rest, Rs}, Drv, _Shell, _Ls, _Encoding) ->
     send_drv_reqs(Drv, Rs),
     {done, Cont, Rest};
-get_line1({undefined,{_A, Mode, Char}, _Cs, Cont, Rs}, Drv, Shell, Ls0, Encoding)
-  when Mode =:= none, Char =:= $\^O;
-       Mode =:= meta, Char =:= $o ->
+get_line1({open_editor, _Cs, Cont, Rs}, Drv, Shell, Ls0, Encoding) ->
     send_drv_reqs(Drv, Rs),
     Buffer = edlin:current_line(Cont),
     send_drv(Drv, {open_editor, Buffer}),
     receive
-        {Drv, {editor_data, Cs}} ->
+        {Drv, {editor_data, Cs1}} ->
             send_drv_reqs(Drv, edlin:erase_line()),
             {more_chars,NewCont,NewRs} = edlin:start(edlin:prompt(Cont)),
             send_drv_reqs(Drv, NewRs),
-            get_line1(edlin:edit_line(Cs, NewCont), Drv, Shell, Ls0, Encoding)
+            get_line1(edlin:edit_line(Cs1, NewCont), Drv, Shell, Ls0, Encoding)
     end;
 %% Move Up, Down in History: Ctrl+P, Ctrl+N
-get_line1({undefined,{_A,Mode,Char},Cs,Cont,Rs}, Drv, Shell, Ls0, Encoding)
-  when Mode =:= none, Char =:= $\^P;
-       Mode =:= meta_left_sq_bracket, Char =:= $A ->
+get_line1({history_up,Cs,Cont,Rs}, Drv, Shell, Ls0, Encoding) ->
     send_drv_reqs(Drv, Rs),
     case up_stack(save_line(Ls0, edlin:current_line(Cont))) of
         {none,_Ls} ->
@@ -622,9 +625,7 @@ get_line1({undefined,{_A,Mode,Char},Cs,Cont,Rs}, Drv, Shell, Ls0, Encoding)
                 Ncont),
               Drv, Shell, Ls, Encoding)
     end;
-get_line1({undefined,{_A,Mode,Char},Cs,Cont,Rs}, Drv, Shell, Ls0, Encoding)
-  when Mode =:= none, Char =:= $\^N;
-       Mode =:= meta_left_sq_bracket, Char =:= $B ->
+get_line1({history_down,Cs,Cont,Rs}, Drv, Shell, Ls0, Encoding) ->
     send_drv_reqs(Drv, Rs),
     case down_stack(save_line(Ls0, edlin:current_line(Cont))) of
         {none,_Ls} ->
@@ -652,14 +653,13 @@ get_line1({undefined,{_A,Mode,Char},Cs,Cont,Rs}, Drv, Shell, Ls0, Encoding)
 %% new modes: search, search_quit, search_found. These are added to
 %% the regular ones (none, meta_left_sq_bracket) and handle special
 %% cases of history search.
-get_line1({undefined,{_A,Mode,Char},Cs,Cont,Rs}, Drv, Shell, Ls, Encoding)
-  when Mode =:= none, Char =:= $\^R ->
+get_line1({search,Cs,Cont,Rs}, Drv, Shell, Ls, Encoding) ->
     send_drv_reqs(Drv, Rs),
     %% drop current line, move to search mode. We store the current
     %% prompt ('N>') and substitute it with the search prompt.
     put(search_quit_prompt, Cont),
     Pbs = prompt_bytes("\033[;1;4msearch:\033[0m ", Encoding),
-    {more_chars,Ncont,_Nrs} = edlin:start(Pbs, search),
+    {more_chars,Ncont,_Nrs} = edlin:start(Pbs, {search,none}),
     get_line1(edlin:edit_line1(Cs, Ncont), Drv, Shell, Ls, Encoding);
 get_line1({Expand, Before, Cs0, Cont,Rs}, Drv, Shell, Ls0, Encoding)
   when Expand =:= expand; Expand =:= expand_full ->
@@ -716,15 +716,10 @@ get_line1({Expand, Before, Cs0, Cont,Rs}, Drv, Shell, Ls0, Encoding)
                  end
          end,
     get_line1(edlin:edit_line(Cs, Cont), Drv, Shell, Ls0, Encoding);
-get_line1({undefined, {_, search_quit, _}, _Cs, _Cont={line, P, Line, none}, Rs}, Drv, Shell, Ls, Encoding) ->
-    get_line1({more_chars, {line, P, Line, search_quit}, Rs}, Drv, Shell, Ls, Encoding);
-get_line1({undefined,_Char,Cs,Cont,Rs}, Drv, Shell, Ls, Encoding) ->
-    send_drv_reqs(Drv, Rs),
-    send_drv(Drv, beep),
-    get_line1(edlin:edit_line(Cs, Cont), Drv, Shell, Ls, Encoding);
+
 %% The search item was found and accepted (new line entered on the exact
 %% result found)
-get_line1({_What,{line,_,_Drv,search_found},Rs}, Drv, Shell, Ls0, Encoding) ->
+get_line1({search_found,_Cs,_,Rs}, Drv, Shell, Ls0, Encoding) ->
     SearchResult = get(search_result),
     LineCont = case SearchResult of
                    [] -> {[],{[],[]},[]};
@@ -734,14 +729,12 @@ get_line1({_What,{line,_,_Drv,search_found},Rs}, Drv, Shell, Ls0, Encoding) ->
     Prompt = edlin:prompt(get(search_quit_prompt)),
     send_drv_reqs(Drv, Rs),
     send_drv_reqs(Drv, edlin:erase_line()),
-    send_drv_reqs(Drv, edlin:redraw_line({line, Prompt, LineCont, none})),
+    send_drv_reqs(Drv, edlin:redraw_line({line, Prompt, LineCont, {normal,none}})),
     put(search_result, []),
-    %% TODO, do even need to save it, won't it be saved by handling {done...}?
-    Ls = save_line(new_stack(get_lines(Ls0)), edlin:current_line({line, edlin:prompt(get(search_quit_prompt)), LineCont, none})),
-    get_line1({done, LineCont, "\n", Rs}, Drv, Shell, Ls, Encoding);
+    get_line1({done, LineCont, "\n", Rs}, Drv, Shell, Ls0, Encoding);
 %% The search mode has been exited, but the user wants to remain in line
 %% editing mode wherever that was, but editing the search result.
-get_line1({What,{line,_,_,search_quit},Rs}, Drv, Shell, Ls, Encoding) ->
+get_line1({search_quit,_Cs,_,Rs}, Drv, Shell, Ls, Encoding) ->
     %% Load back the old prompt with the correct line number.
     case edlin:prompt(get(search_quit_prompt)) of
         Prompt -> % redraw the line and keep going with the same stack position
@@ -751,20 +744,20 @@ get_line1({What,{line,_,_,search_quit},Rs}, Drv, Shell, Ls, Encoding) ->
                     _  -> [Last|LB] = lists:reverse(SearchResult),
                           {LB, {lists:reverse(Last), []}, []}
                 end,
-            NCont = {line,Prompt,L,none},
+            NCont = {line,Prompt,L,{normal,none}},
             put(search_result, []),
             send_drv_reqs(Drv, [delete_line|Rs]),
             send_drv_reqs(Drv, edlin:redraw_line(NCont)),
-            get_line1({What, NCont ,[]}, Drv, Shell, pad_stack(Ls), Encoding)
+            get_line1({more_chars, NCont ,[]}, Drv, Shell, pad_stack(Ls), Encoding)
     end;
-get_line1({What,_Cont={line,_,_,search_cancel},Rs}, Drv, Shell, Ls, Encoding) ->
+get_line1({search_cancel,_Cs,_,Rs}, Drv, Shell, Ls, Encoding) ->
     NCont = get(search_quit_prompt),
     put(search_result, []),
     send_drv_reqs(Drv, [delete_line|Rs]),
     send_drv_reqs(Drv, edlin:redraw_line(NCont)),
-    get_line1({What, NCont, []}, Drv, Shell, Ls, Encoding);
+    get_line1({more_chars, NCont, []}, Drv, Shell, Ls, Encoding);
 %% Search mode is entered.
-get_line1({What,{line,Prompt,{_,{RevCmd0,_},_},search},_Rs},
+get_line1({What,{line,Prompt,{_,{RevCmd0,_},_},{search, none}},_Rs},
           Drv, Shell, Ls0, Encoding) ->
     %% Figure out search direction. ^S and ^R are returned through edlin
     %% whenever we received a search while being already in search mode.
@@ -798,7 +791,7 @@ get_line1({What,{line,Prompt,{_,{RevCmd0,_},_},search},_Rs},
                              send_drv(Drv, {put_expand_no_trim, unicode, unicode:characters_to_binary(Output)}),
                              {Ls2, {[],{RevCmd, []},[]}}
                      end,
-    Cont = {line,Prompt,NewStack,search},
+    Cont = {line,Prompt,NewStack,{search, none}},
     more_data(What, Cont, Drv, Shell, Ls, Encoding);
 get_line1({What,Cont0,Rs}, Drv, Shell, Ls, Encoding) ->
     send_drv_reqs(Drv, Rs),
@@ -810,7 +803,8 @@ more_data(What, Cont0, Drv, Shell, Ls, Encoding) ->
             send_drv_reqs(Drv, edlin:redraw_line(Cont0)),
             more_data(What, Cont0, Drv, Shell, Ls, Encoding);
         {Drv,{data,Cs}} ->
-            get_line1(edlin:edit_line(cast(Cs, list), Cont0),
+            Res = edlin:edit_line(cast(Cs, list), Cont0),
+            get_line1(Res,
                       Drv, Shell, Ls, Encoding);
         {Drv,eof} ->
             get_line1(edlin:edit_line(eof, Cont0), Drv, Shell, Ls, Encoding);
@@ -835,11 +829,25 @@ more_data(What, Cont0, Drv, Shell, Ls, Encoding) ->
             get_line1(edlin:edit_line([], Cont0), Drv, Shell, Ls, Encoding)
     end.
 
-get_line_echo_off(Chars, Pbs, Drv, Shell) ->
+get_line_echo_off(Chars, ToEnc, Pbs, Drv, Shell) ->
     send_drv_reqs(Drv, [{put_chars, unicode,Pbs}]),
-    get_line_echo_off1(edit_line(Chars,[]), Drv, Shell).
+    case get_line_echo_off1(edit_line(Chars,[]), Drv, Shell) of
+        {done, Line, _Rest} = Res when ToEnc =:= latin1 ->
+            case check_encoding(Line, ToEnc) of
+                false ->
+                    {no_translation, unicode, ToEnc};
+                true ->
+                    Res
+            end;
+        Res ->
+            Res
+    end.
 
-get_line_echo_off1({Chars,[]}, Drv, Shell) ->
+get_line_echo_off1({Chars,[],Rs}, Drv, Shell) ->
+    case get(echo) of
+        true -> send_drv_reqs(Drv, Rs);
+        false -> skip
+    end,
     receive
 	{Drv,{data,Cs}} ->
 	    get_line_echo_off1(edit_line(cast(Cs, list), Chars), Drv, Shell);
@@ -847,11 +855,11 @@ get_line_echo_off1({Chars,[]}, Drv, Shell) ->
 	    get_line_echo_off1(edit_line(eof, Chars), Drv, Shell);
 	{io_request,From,ReplyAs,Req} when is_pid(From) ->
 	    io_request(Req, From, ReplyAs, Drv, Shell, []),
-	    get_line_echo_off1({Chars,[]}, Drv, Shell);
+	    get_line_echo_off1({Chars,[],[]}, Drv, Shell);
         {reply,{From,ReplyAs},Reply} when From =/= undefined ->
             %% We take care of replies from puts here as well
             io_reply(From, ReplyAs, Reply),
-            get_line_echo_off1({Chars,[]},Drv, Shell);
+            get_line_echo_off1({Chars,[],[]},Drv, Shell);
 	{'EXIT',Drv,interrupt} ->
 	    interrupted;
 	{'EXIT',Drv,_} ->
@@ -861,9 +869,12 @@ get_line_echo_off1({Chars,[]}, Drv, Shell) ->
     end;
 get_line_echo_off1(eof, _Drv, _Shell) ->
     {done,eof,eof};
-get_line_echo_off1({Chars,Rest}, _Drv, _Shell) ->
+get_line_echo_off1({Chars,Rest,Rs}, Drv, _Shell) ->
+    case get(echo) of
+        true -> send_drv_reqs(Drv, Rs);
+        false -> skip
+    end,
     {done,lists:reverse(Chars),case Rest of done -> []; _ -> Rest end}.
-
 get_chars_echo_off(Pbs, Drv, Shell) ->
     send_drv_reqs(Drv, [{insert_chars, unicode,Pbs}]),
     get_chars_echo_off1(Drv, Shell).
@@ -898,22 +909,26 @@ get_chars_echo_off1(Drv, Shell) ->
 %% - ^d in posix/icanon mode: eof, delete-forward in edlin
 %% - ^r in posix/icanon mode: reprint (silly in echo-off mode :-))
 %% - ^w in posix/icanon mode: word-erase (produces a beep in edlin)
-edit_line(eof, []) ->
+edit_line(Input, State) ->
+    edit_line(Input, State, []).
+edit_line(eof, [], _) ->
     eof;
-edit_line(eof, Chars) ->
-    {Chars,eof};
-edit_line([],Chars) ->
-    {Chars,[]};
-edit_line([$\r,$\n|Cs],Chars) ->
-    {[$\n | Chars], remainder_after_nl(Cs)};
-edit_line([NL|Cs],Chars) when NL =:= $\r; NL =:= $\n ->
-    {[$\n | Chars], remainder_after_nl(Cs)};
-edit_line([Erase|Cs],[]) when Erase =:= $\177; Erase =:= $\^H ->
-    edit_line(Cs,[]);
-edit_line([Erase|Cs],[_|Chars]) when Erase =:= $\177; Erase =:= $\^H ->
-    edit_line(Cs,Chars);
-edit_line([Char|Cs],Chars) ->
-    edit_line(Cs,[Char|Chars]).
+edit_line(eof, Chars, Rs) ->
+    {Chars,eof, lists:reverse(Rs)};
+edit_line([],Chars, Rs) ->
+    {Chars,[],lists:reverse(Rs)};
+edit_line([$\r,$\n|Cs],Chars, Rs) ->
+    {[$\n | Chars], remainder_after_nl(Cs), lists:reverse([{put_chars, unicode, "\n"}|Rs])};
+edit_line([NL|Cs],Chars, Rs) when NL =:= $\r; NL =:= $\n ->
+    {[$\n | Chars], remainder_after_nl(Cs), lists:reverse([{put_chars, unicode, "\n"}|Rs])};
+edit_line([Erase|Cs],[], Rs) when Erase =:= $\177; Erase =:= $\^H ->
+    edit_line(Cs,[], Rs);
+edit_line([Erase|Cs],[_|Chars], Rs) when Erase =:= $\177; Erase =:= $\^H ->
+    edit_line(Cs,Chars, [{delete_chars, -1}|Rs]);
+edit_line([CtrlChar|Cs],Chars, Rs) when CtrlChar < 32 ->
+    edit_line(Cs,Chars,Rs);
+edit_line([Char|Cs],Chars, Rs) ->
+    edit_line(Cs,[Char|Chars], [{put_chars, unicode, [Char]}|Rs]).
 
 remainder_after_nl("") -> done;
 remainder_after_nl(Cs) -> Cs.
@@ -1074,3 +1089,18 @@ append(eof, L, _) ->
     L;
 append(B, L, FromEnc) ->
     unicode:characters_to_list(B, FromEnc) ++ L.
+
+check_encoding(eof, _) ->
+    true;
+check_encoding(ListOrBinary, unicode) when is_list(ListOrBinary); is_binary(ListOrBinary) ->
+    true;
+check_encoding(List, latin1) when is_list(List) ->
+    is_latin1(List).
+
+is_latin1([H|T]) when 0 =< H, H =< 255 ->
+    is_latin1(T);
+is_latin1([]) ->
+    true;
+is_latin1(_) ->
+    false.
+
