@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2020. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@
 %%
 
 %%----------------------------------------------------------------------
-%% Purpose: Help funtions for handling the TLS (specific parts of)
+%% Purpose: Help functions for handling the TLS (specific parts of)
 %%% SSL/TLS/DTLS handshake protocol
 %%----------------------------------------------------------------------
 
@@ -36,13 +36,16 @@
 -include_lib("kernel/include/logger.hrl").
 
 %% Handshake handling
--export([client_hello/9, hello/5, hello/4]).
+-export([client_hello/11, hello/5, hello/4]).
 
 %% Handshake encoding
 -export([encode_handshake/2]).
 
-%% Handshake decodeing
--export([get_tls_handshake/4, decode_handshake/3]).
+%% Handshake decoding
+-export([get_tls_handshakes/4, decode_handshake/3]).
+
+%% Handshake helper
+-export([ocsp_nonce/1]).
 
 -type tls_handshake() :: #client_hello{} | ssl_handshake:ssl_handshake().
 
@@ -51,8 +54,9 @@
 %%====================================================================
 %%--------------------------------------------------------------------
 -spec client_hello(ssl:host(), inet:port_number(), ssl_record:connection_states(),
-		   ssl_options(), binary(), boolean(), der_cert(),
-                   #key_share_client_hello{} | undefined, tuple() | undefined) ->
+		   ssl_options(), binary(), boolean(),
+                   #key_share_client_hello{} | undefined, tuple() | undefined,
+                   binary() | undefined, db_handle() | undefined, certdb_ref() | undefined) ->
 			  #client_hello{}.
 %%
 %% Description: Creates a client hello message.
@@ -62,7 +66,7 @@ client_hello(_Host, _Port, ConnectionStates,
                ciphers := UserSuites,
                fallback := Fallback
               } = SslOpts,
-	     Id, Renegotiation, _OwnCert, KeyShare, TicketData) ->
+	     Id, Renegotiation, KeyShare, TicketData, OcspNonce, CertDbHandle, CertDbRef) ->
     Version = tls_record:highest_protocol_version(Versions),
 
     %% In TLS 1.3, the client indicates its version preferences in the
@@ -70,21 +74,23 @@ client_hello(_Host, _Port, ConnectionStates,
     %% legacy_version field MUST be set to 0x0303, which is the version
     %% number for TLS 1.2.
     LegacyVersion =
-        case tls_record:is_higher(Version, {3,2}) of
+        case tls_record:is_higher(Version, ?TLS_1_1) of
             true ->
-                {3,3};
+                ?TLS_1_2;
             false ->
                 Version
         end,
-    #{security_parameters := SecParams} = 
+    #{security_parameters := SecParams} =
         ssl_record:pending_connection_state(ConnectionStates, read),
-    AvailableCipherSuites = ssl_handshake:available_suites(UserSuites, Version),     
-    Extensions = ssl_handshake:client_hello_extensions(Version, 
+    AvailableCipherSuites = ssl_handshake:available_suites(UserSuites, Version),
+    Extensions = ssl_handshake:client_hello_extensions(Version,
 						       AvailableCipherSuites,
-						       SslOpts, ConnectionStates, 
+						       SslOpts,
+                                                       ConnectionStates,
                                                        Renegotiation,
                                                        KeyShare,
-                                                       TicketData),
+                                                       TicketData,
+                                                       OcspNonce, CertDbHandle, CertDbRef),
     CipherSuites = ssl_handshake:cipher_suites(AvailableCipherSuites, Renegotiation, Fallback),
     #client_hello{session_id = Id,
 		  client_version = LegacyVersion,
@@ -100,12 +106,10 @@ client_hello(_Host, _Port, ConnectionStates,
 				    atom(), ssl_record:connection_states(), 
 				    binary() | undefined, ssl:kex_algo()},
 	    boolean(), #session{}) ->
-		   {tls_record:tls_version(), ssl:session_id(), 
-		    ssl_record:connection_states(), alpn | npn, binary() | undefined}|
-		   {tls_record:tls_version(), {resumed | new, #session{}}, 
-		    ssl_record:connection_states(), binary() | undefined, 
-                    HelloExt::map(), {ssl:hash(), ssl:sign_algo()} | 
-                    undefined} | {atom(), atom()} | {atom(), atom(), tuple()} | #alert{}.
+          {tls_record:tls_version(), ssl:session_id(), 
+           ssl_record:connection_states(), alpn | npn, binary() | undefined, map()}|
+          {atom(), atom(), tls_record:tls_version(), map()}.
+                                                % Otherwise Throws #alert{}
 %%
 %% Description: Handles a received hello message
 %%--------------------------------------------------------------------
@@ -125,7 +129,7 @@ hello(#server_hello{server_version = {Major, Minor},
        (M > 3 orelse M =:= 3 andalso N >= 4) andalso  %% TLS 1.3 client
        (Major =:= 3 andalso Minor < 3 andalso         %% Negotiating TLS 1.1 or prior
         Down =:= ?RANDOM_OVERRIDE_TLS11) ->
-    ?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER);
+    throw(?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER));
 
 %% TLS 1.2 clients SHOULD also check that the last eight bytes are not
 %% equal to the second value if the ServerHello indicates TLS 1.1 or below.
@@ -135,7 +139,7 @@ hello(#server_hello{server_version = {Major, Minor},
   when (M =:= 3 andalso N =:= 3) andalso              %% TLS 1.2 client
        (Major =:= 3 andalso Minor < 3 andalso         %% Negotiating TLS 1.1 or prior
         Down =:= ?RANDOM_OVERRIDE_TLS11) ->
-    ?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER);
+    throw(?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER));
 
 
 %% TLS 1.3 - 4.2.1.  Supported Versions
@@ -153,24 +157,26 @@ hello(#server_hello{server_version = LegacyVersion,
 		    cipher_suite = CipherSuite,
 		    compression_method = Compression,
 		    session_id = SessionId,
-                    extensions = #{server_hello_selected_version :=
-                                       #server_hello_selected_version{selected_version = Version} = HelloExt}
-                   },
+                    extensions =
+                        #{server_hello_selected_version :=
+                              #server_hello_selected_version{
+                                 selected_version = Version}} = HelloExt},
       #{versions := SupportedVersions} = SslOpt,
       ConnectionStates0, Renegotiation, OldId) ->
+    Stapling = maps:get(ocsp_stapling, SslOpt, ?DEFAULT_OCSP_STAPLING),
     %% In TLS 1.3, the TLS server indicates its version using the "supported_versions" extension
     %% (Section 4.2.1), and the legacy_version field MUST be set to 0x0303, which is the version
     %% number for TLS 1.2.
     %% The "supported_versions" extension is supported from TLS 1.2.
-    case LegacyVersion > {3,3} orelse
-        LegacyVersion =:= {3,3} andalso Version < {3,3} of
+    case ?TLS_LT(LegacyVersion, ?TLS_1_2) orelse
+        LegacyVersion =:= ?TLS_1_2 andalso ?TLS_LT(Version,  ?TLS_1_2) of
         true ->
-            ?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER);
+            throw(?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER));
         false ->
             case tls_record:is_acceptable_version(Version, SupportedVersions) of
                 true ->
                     case Version of
-                        {3,3} ->
+                        ?TLS_1_2 ->
                             IsNew = ssl_session:is_new(OldId, SessionId),
                             %% TLS 1.2 ServerHello with "supported_versions" (special case)
                             handle_server_hello_extensions(Version, SessionId, Random, CipherSuite,
@@ -178,10 +184,12 @@ hello(#server_hello{server_version = LegacyVersion,
                                                            ConnectionStates0, Renegotiation, IsNew);
                         SelectedVersion ->
                             %% TLS 1.3
-                            {next_state, wait_sh, SelectedVersion}
+                            {next_state, wait_sh, SelectedVersion,
+                             #{ocsp_stapling => Stapling,
+                               ocsp_expect => ocsp_expect(Stapling)}}
                     end;
                 false ->
-                    ?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER)
+                    throw(?ALERT_REC(?FATAL, ?ILLEGAL_PARAMETER))
             end
     end;
 
@@ -200,22 +208,21 @@ hello(#server_hello{server_version = Version,
 					   Compression, HelloExt, SslOpt, 
                                            ConnectionStates0, Renegotiation, IsNew);
 	false ->
-	    ?ALERT_REC(?FATAL, ?PROTOCOL_VERSION)
+	    throw(?ALERT_REC(?FATAL, ?PROTOCOL_VERSION))
     end.
 
 
 %%--------------------------------------------------------------------
 -spec hello(#client_hello{}, ssl_options(),
-	    ssl_record:connection_states() | {inet:port_number(), #session{}, db_handle(),
-				    atom(), ssl_record:connection_states(), 
-				    binary() | undefined, ssl:kex_algo()},
+	    {pid(), #session{}, ssl_record:connection_states(),
+             list(), ssl:kex_algo()},
 	    boolean()) ->
 		   {tls_record:tls_version(), ssl:session_id(), 
 		    ssl_record:connection_states(), alpn | npn, binary() | undefined}|
 		   {tls_record:tls_version(), {resumed | new, #session{}}, 
 		    ssl_record:connection_states(), binary() | undefined, 
                     HelloExt::map(), {ssl:hash(), ssl:sign_algo()} | 
-                    undefined} | {atom(), atom()} | {atom(), atom(), tuple()} | #alert{}.
+                    undefined} | {atom(), atom()} | {atom(), atom(), tuple()}.
 %% TLS 1.2 Server
 %% - If "supported_versions" is present (ClientHello):
 %%   - Select version from "supported_versions" (ignore ClientHello.legacy_version)
@@ -241,10 +248,11 @@ hello(#client_hello{client_version = _ClientVersion,
         Version = ssl_handshake:select_supported_version(ClientVersions, Versions),
         do_hello(Version, Versions, CipherSuites, Hello, SslOpts, Info, Renegotiation)
     catch
-	_:_ ->
-	    ?ALERT_REC(?FATAL, ?HANDSHAKE_FAILURE, malformed_handshake_data)
+	error:Reason:ST ->
+            ?SSL_LOG(info, handshake_error, [{reason,Reason}, {stacktrace, ST}]),
+	    throw(?ALERT_REC(?FATAL, ?HANDSHAKE_FAILURE, malformed_handshake_data))
     end;
-			       
+
 hello(#client_hello{client_version = ClientVersion,
 		    cipher_suites = CipherSuites} = Hello,
       #{versions := Versions} = SslOpts,
@@ -254,12 +262,12 @@ hello(#client_hello{client_version = ClientVersion,
         do_hello(Version, Versions, CipherSuites, Hello, SslOpts, Info, Renegotiation)
     catch
         error:{case_clause,{asn1, Asn1Reason}} ->
-	    %% ASN-1 decode of certificate somehow failed
-            ?ALERT_REC(?FATAL, ?INTERNAL_ERROR, {failed_to_decode_own_certificate, Asn1Reason});
-	_:_ ->
-	    ?ALERT_REC(?FATAL, ?HANDSHAKE_FAILURE, malformed_handshake_data)
-    end.  
-
+            %% ASN-1 decode of certificate somehow failed
+            throw(?ALERT_REC(?FATAL, ?INTERNAL_ERROR, {failed_to_decode_own_certificate, Asn1Reason}));
+        error:Reason:ST ->
+            ?SSL_LOG(info, handshake_error, [{reason,Reason}, {stacktrace, ST}]),
+            throw(?ALERT_REC(?FATAL, ?HANDSHAKE_FAILURE, malformed_handshake_data))
+    end.
 
 %%--------------------------------------------------------------------
 %%% Handshake encodeing
@@ -278,11 +286,11 @@ encode_handshake(Package, Version) ->
 
 
 %%--------------------------------------------------------------------
-%%% Handshake decodeing
+%%% Handshake decoding
 %%--------------------------------------------------------------------
 
 %%--------------------------------------------------------------------
--spec get_tls_handshake(tls_record:tls_version(), binary(), binary() | iolist(), 
+-spec get_tls_handshakes(tls_record:tls_version(), binary(), binary() | iolist(),
                         ssl_options()) ->
      {[{tls_handshake(), binary()}], binary()}.
 %%
@@ -290,10 +298,27 @@ encode_handshake(Package, Version) ->
 %% and returns it as a list of handshake messages, also returns leftover
 %% data.
 %%--------------------------------------------------------------------
-get_tls_handshake(Version, Data, <<>>, Options) ->
-    get_tls_handshake_aux(Version, Data, Options, []);
-get_tls_handshake(Version, Data, Buffer, Options) ->
-    get_tls_handshake_aux(Version, list_to_binary([Buffer, Data]), Options, []).
+get_tls_handshakes(Version, Data, <<>>, Options) ->
+    get_tls_handshakes_aux(Version, Data, Options, []);
+get_tls_handshakes(Version, Data, Buffer, Options) ->
+    get_tls_handshakes_aux(Version, list_to_binary([Buffer, Data]), Options, []).
+
+%%--------------------------------------------------------------------
+%%% Handshake helper
+%%--------------------------------------------------------------------
+
+%%--------------------------------------------------------------------
+-spec ocsp_nonce(map()) -> binary() | undefined.
+%%
+%% Description: Get an OCSP nonce
+%%--------------------------------------------------------------------
+ocsp_nonce(SslOpts) ->
+    case maps:get(ocsp_stapling, SslOpts, disabled) of
+        #{ocsp_nonce := true} ->
+            public_key:der_encode('Nonce', crypto:strong_rand_bytes(8));
+        _ ->
+            undefined
+    end.
 
 %%--------------------------------------------------------------------
 %%% Internal functions
@@ -305,85 +330,82 @@ handle_client_hello(Version,
                                   random = Random,
                                   extensions = HelloExt},
 		    #{versions := Versions,
-                      signature_algs := SupportedHashSigns,
                       eccs := SupportedECCs,
                       honor_ecc_order := ECCOrder} = SslOpts,
-		    {Port, Session0, Cache, CacheCb, ConnectionStates0, Cert, _}, 
+		    {SessIdTracker, Session0, ConnectionStates0, CertKeyPairs, _},
                     Renegotiation) ->
     case tls_record:is_acceptable_version(Version, Versions) of
 	true ->
+            SupportedHashSigns = supported_hashsigns(maps:get(signature_algs, SslOpts, undefined)),
             Curves = maps:get(elliptic_curves, HelloExt, undefined),
-            ClientHashSigns = maps:get(signature_algs, HelloExt, undefined),
-            ClientSignatureSchemes = maps:get(signature_algs_cert, HelloExt, undefined),
+            ClientHashSigns = get_signature_ext(signature_algs, HelloExt, Version),
+            ClientSignatureSchemes = get_signature_ext(signature_algs_cert, HelloExt, Version),
 	    AvailableHashSigns = ssl_handshake:available_signature_algs(
-				   ClientHashSigns, SupportedHashSigns, Cert, Version),
+				   ClientHashSigns, SupportedHashSigns, Version),
 	    ECCCurve = ssl_handshake:select_curve(Curves, SupportedECCs, ECCOrder),
-	    {Type, #session{cipher_suite = CipherSuite} = Session1}
-		= ssl_handshake:select_session(SugesstedId, CipherSuites, 
+	    {Type, #session{cipher_suite = CipherSuite,
+                            own_certificates = [OwnCert |_]} = Session1}
+		= ssl_handshake:select_session(SugesstedId, CipherSuites,
                                                AvailableHashSigns, Compressions,
-					       Port, Session0#session{ecc = ECCCurve}, 
-                                               Version, SslOpts, Cache, CacheCb, Cert),
-	    case CipherSuite of 
+					       SessIdTracker, Session0#session{ecc = ECCCurve},
+                                               Version, SslOpts, CertKeyPairs),
+	    case CipherSuite of
 		no_suite ->
-                    ?ALERT_REC(?FATAL, ?INSUFFICIENT_SECURITY, no_suitable_ciphers);
+                    throw(?ALERT_REC(?FATAL, ?INSUFFICIENT_SECURITY, no_suitable_ciphers));
 		_ ->
 		    #{key_exchange := KeyExAlg} = ssl_cipher_format:suite_bin_to_map(CipherSuite),
-		    case ssl_handshake:select_hashsign({ClientHashSigns, ClientSignatureSchemes},
-                                                       Cert, KeyExAlg,
+                    case ssl_handshake:select_hashsign({ClientHashSigns, ClientSignatureSchemes},
+                                                       OwnCert, KeyExAlg,
                                                        SupportedHashSigns,
                                                        Version) of
 			#alert{} = Alert ->
-			    Alert;
+			    throw(Alert);
 			HashSign ->
 			    handle_client_hello_extensions(Version, Type, Random, 
                                                            CipherSuites, HelloExt,
 							   SslOpts, Session1, 
                                                            ConnectionStates0,
-							   Renegotiation, HashSign)
+                                                           Renegotiation, HashSign)
 		    end
 	    end;
 	false ->
-	    ?ALERT_REC(?FATAL, ?PROTOCOL_VERSION)
+	    throw(?ALERT_REC(?FATAL, ?PROTOCOL_VERSION))
     end.
 
+supported_hashsigns(undefined) ->
+    undefined;
+supported_hashsigns(SigAlgs) ->
+    ssl_cipher:signature_schemes_1_2(SigAlgs).
+
 handle_client_hello_extensions(Version, Type, Random, CipherSuites,
                                HelloExt, SslOpts, Session0, ConnectionStates0, 
                                Renegotiation, HashSign) ->
-    try ssl_handshake:handle_client_hello_extensions(tls_record, Random, CipherSuites,
-						     HelloExt, Version, SslOpts,
-						     Session0, ConnectionStates0, 
+    {Session, ConnectionStates, Protocol, ServerHelloExt} =
+        ssl_handshake:handle_client_hello_extensions(tls_record, Random, CipherSuites,
+                                                     HelloExt, Version, SslOpts,
+                                                     Session0, ConnectionStates0,
                                                      Renegotiation,
-                                                     Session0#session.is_resumable) of
-	{Session, ConnectionStates, Protocol, ServerHelloExt} ->
-	    {Version, {Type, Session}, ConnectionStates, Protocol, 
-             ServerHelloExt, HashSign}
-    catch throw:Alert ->
-	    Alert
-    end.
-
+                                                     Session0#session.is_resumable),
+    {Version, {Type, Session}, ConnectionStates, Protocol, ServerHelloExt, HashSign}.
 
 handle_server_hello_extensions(Version, SessionId, Random, CipherSuite,
-			Compression, HelloExt, SslOpt, ConnectionStates0, Renegotiation, IsNew) ->
-    try ssl_handshake:handle_server_hello_extensions(tls_record, Random, CipherSuite,
-						      Compression, HelloExt, Version,
-						      SslOpt, ConnectionStates0, 
-                                                     Renegotiation, IsNew) of
-	{ConnectionStates, ProtoExt, Protocol} ->
-	    {Version, SessionId, ConnectionStates, ProtoExt, Protocol}
-    catch throw:Alert ->
-	    Alert
-    end.
-
+                               Compression, HelloExt, SslOpt, ConnectionStates0, Renegotiation, IsNew) ->
+    {ConnectionStates, ProtoExt, Protocol, OcspState} =
+        ssl_handshake:handle_server_hello_extensions(tls_record, Random, CipherSuite,
+                                                     Compression, HelloExt, Version,
+                                                     SslOpt, ConnectionStates0,
+                                                     Renegotiation, IsNew),
+    {Version, SessionId, ConnectionStates, ProtoExt, Protocol, OcspState}.
 
 do_hello(undefined, _Versions, _CipherSuites, _Hello, _SslOpts, _Info, _Renegotiation) ->
-    ?ALERT_REC(?FATAL, ?PROTOCOL_VERSION);
+    throw(?ALERT_REC(?FATAL, ?PROTOCOL_VERSION));
 do_hello(Version, Versions, CipherSuites, Hello, SslOpts, Info, Renegotiation) ->
     case ssl_cipher:is_fallback(CipherSuites) of
         true ->
             Highest = tls_record:highest_protocol_version(Versions),
             case tls_record:is_higher(Highest, Version) of
                 true ->
-                    ?ALERT_REC(?FATAL, ?INAPPROPRIATE_FALLBACK);
+                    throw(?ALERT_REC(?FATAL, ?INAPPROPRIATE_FALLBACK));
                 false ->
                     handle_client_hello(Version, Hello, SslOpts, Info, Renegotiation)
             end;
@@ -392,9 +414,9 @@ do_hello(Version, Versions, CipherSuites, Hello, SslOpts, Info, Renegotiation) -
     end.
 
 %%--------------------------------------------------------------------
-enc_handshake(#hello_request{}, {3, N}) when N < 4 ->
+enc_handshake(#hello_request{}, Version) when ?TLS_LT(Version, ?TLS_1_3)->
     {?HELLO_REQUEST, <<>>};
-enc_handshake(#client_hello{client_version = {Major, Minor} = Version,
+enc_handshake(#client_hello{client_version = ServerVersion,
 		     random = Random,
 		     session_id = SessionID,
 		     cipher_suites = CipherSuites,
@@ -405,36 +427,37 @@ enc_handshake(#client_hello{client_version = {Major, Minor} = Version,
     CmLength = byte_size(BinCompMethods),
     BinCipherSuites = list_to_binary(CipherSuites),
     CsLength = byte_size(BinCipherSuites),
-    ExtensionsBin = ssl_handshake:encode_hello_extensions(HelloExtensions, Version),
+    ExtensionsBin = ssl_handshake:encode_hello_extensions(HelloExtensions),
+    {Major,Minor} = ServerVersion,
 
     {?CLIENT_HELLO, <<?BYTE(Major), ?BYTE(Minor), Random:32/binary,
 		      ?BYTE(SIDLength), SessionID/binary,
 		      ?UINT16(CsLength), BinCipherSuites/binary,
 		      ?BYTE(CmLength), BinCompMethods/binary, ExtensionsBin/binary>>};
-enc_handshake(HandshakeMsg, {3, 4}) ->
+enc_handshake(HandshakeMsg, ?TLS_1_3) ->
     tls_handshake_1_3:encode_handshake(HandshakeMsg);
 enc_handshake(HandshakeMsg, Version) ->
     ssl_handshake:encode_handshake(HandshakeMsg, Version).
 
 %%--------------------------------------------------------------------
-get_tls_handshake_aux(Version, <<?BYTE(Type), ?UINT24(Length),
-				 Body:Length/binary,Rest/binary>>, 
+get_tls_handshakes_aux(Version, <<?BYTE(Type), ?UINT24(Length),
+                                  Body:Length/binary,Rest/binary>>,
                       #{log_level := LogLevel} = Opts,  Acc) ->
     Raw = <<?BYTE(Type), ?UINT24(Length), Body/binary>>,
     try decode_handshake(Version, Type, Body) of
-	Handshake ->
+        Handshake ->
             ssl_logger:debug(LogLevel, inbound, 'handshake', Handshake),
-	    get_tls_handshake_aux(Version, Rest, Opts, [{Handshake,Raw} | Acc])
+            get_tls_handshakes_aux(Version, Rest, Opts, [{Handshake,Raw} | Acc])
     catch
-        throw:#alert{} = Alert ->
-            throw(Alert);
-	_:_ ->
-	    throw(?ALERT_REC(?FATAL, ?HANDSHAKE_FAILURE, handshake_decode_error))
+        error:Reason:ST ->
+            ?SSL_LOG(info, handshake_error, [{reason,Reason}, {stacktrace, ST}]),
+            throw(?ALERT_REC(?FATAL, ?DECODE_ERROR, handshake_decode_error))
     end;
-get_tls_handshake_aux(_Version, Data, _, Acc) ->
+get_tls_handshakes_aux(_Version, Data, _, Acc) ->
     {lists:reverse(Acc), Data}.
 
-decode_handshake({3, N}, ?HELLO_REQUEST, <<>>) when N < 4 ->
+decode_handshake(Version, ?HELLO_REQUEST, <<>>)
+  when ?TLS_LT(Version, ?TLS_1_3) ->
     #hello_request{};
 decode_handshake(Version, ?CLIENT_HELLO,
                  <<?BYTE(Major), ?BYTE(Minor), Random:32/binary,
@@ -453,7 +476,32 @@ decode_handshake(Version, ?CLIENT_HELLO,
        compression_methods = erlang:binary_to_list(Comp_methods),
        extensions = DecodedExtensions
       };
-decode_handshake({3, 4}, Tag, Msg) ->
+decode_handshake(?TLS_1_3, Tag, Msg) ->
     tls_handshake_1_3:decode_handshake(Tag, Msg);
 decode_handshake(Version, Tag, Msg) ->
     ssl_handshake:decode_handshake(Version, Tag, Msg).
+
+
+ocsp_expect(true) ->
+    staple;
+ocsp_expect(_) ->
+    no_staple.
+
+get_signature_ext(Ext, HelloExt, ?TLS_1_2) ->
+    case maps:get(Ext, HelloExt, undefined) of
+        %% Signature algorithms was not sent
+        undefined ->
+            undefined;
+        %% Can happen when connection is upgraded and sni_fun changes
+        %% the versions option from default
+        #signature_algorithms{signature_scheme_list = Schemes} ->
+            #hash_sign_algos{hash_sign_algos = ssl_cipher:signature_schemes_1_2(Schemes)};
+        #signature_algorithms_cert{} = Algos ->
+            Algos;
+        #hash_sign_algos{} = Algos ->
+            Algos
+    end;
+get_signature_ext(Ext, HelloExt, _) ->
+    maps:get(Ext, HelloExt, undefined).
+
+
